@@ -22,7 +22,7 @@ from clu.predicates import attr, allattrs, anyof
 from clu.sanitizer import utf8_encode
 from clu.typology import ispath, isvalidpath
 from .misc import masked_permissions
-from .misc import stringify, suffix_searcher, u8str
+from .misc import stringify, suffix_searcher, swapext, u8str
 from clu.exporting import Exporter
 
 exporter = Exporter(path=__file__)
@@ -558,7 +558,7 @@ class TemporaryName(collections.abc.Hashable,
             if os.path.exists(destination):
                 if os.path.samefile(self._name, destination):
                     raise FilesystemError("Can’t copy to identical locations")
-                if os.path.isdir(self._name, destination):
+                if os.path.isdir(destination):
                     raise FilesystemError("Can’t copy files over a directory")
                 rm_rf(destination)
             return os.path.exists(
@@ -677,10 +677,10 @@ class Directory(collections.abc.Hashable,
             calls will be issued. Values for “pth” can be string-like, or existing
             Directory instances -- either will work.
             
-            There are two decendant classes of Directory (q.v. definitions below)
-            that enforce stipulations for the “pth” parameter: the `cd` class 
+            There are several decendant classes of Directory (q.v. definitions below)
+            that enforce stipulations for the “pth” parameter – e.g. the `cd` class 
             requires a target path to be provided (and therefore will nearly always
-            change the working directory when invoked as a context manager). Its
+            change the working directory when invoked as a context manager); its
             sibling class `wd` forbids the naming of a “pth” value, thereby always
             initializing itself with the current working directory as its target,
             and fundamentally avoids issuing any directory-change calls.
@@ -867,12 +867,9 @@ class Directory(collections.abc.Hashable,
             Specify an optional “suffix” parameter to filter the list by a
             particular file suffix (leading dots unnecessary but unharmful).
         """
-        files = (str(direntry.name) \
-                 for direntry in filter(non_dotfile_matcher,
-                                scandir(self.realpath(pth))))
-        if not suffix:
-            return files
-        return filter(suffix_searcher(suffix), files)
+        return tuple(filter(suffix_searcher(suffix),
+                    (direntry.name for direntry in filter(non_dotfile_matcher,
+                                                  scandir(self.realpath(pth))))))
     
     def ls_la(self, pth=None, suffix=None):
         """ List all files, including files whose name starts with a dot.
@@ -888,11 +885,8 @@ class Directory(collections.abc.Hashable,
             commands I ever learned, and it reads better than `ls_a()` which
             I think looks awkward and goofy.)
         """
-        files = (str(direntry.name) \
-                 for direntry in scandir(self.realpath(pth)))
-        if not suffix:
-            return files
-        return filter(suffix_searcher(suffix), files)
+        return tuple(filter(suffix_searcher(suffix),
+                    (direntry.name for direntry in scandir(self.realpath(pth)))))
     
     def subpath(self, subpth, whence=None, requisite=False):
         """ Returns the path to a subpath of the instances’ target path. """
@@ -927,10 +921,13 @@ class Directory(collections.abc.Hashable,
             raise FilesystemError(str(os_error))
         return self
     
-    def walk(self, followlinks=False):
+    def walk(self, followlinks=True):
         """ Sugar for calling X.walk(self.name), where X is either
             `scandir` (in the case of python 2.7) or `os` (for
             python 3 and thereafter).
+            
+            Note that the “followlinks” default here is True, whereas
+            the underlying functions default to False for that argument.
         """
         return walk(self.name, followlinks=followlinks)
     
@@ -951,12 +948,11 @@ class Directory(collections.abc.Hashable,
             Used internally in the implementations of the instance
             methods “Directory.flatten(…)”, and “Directory.zip_archive(…)”.
         """
-        return os.path.relpath(path,
-              start=os.path.abspath(
-                    os.path.join(self.name,
-                    os.pardir)))
+        return os.path.relpath(path, start=os.path.abspath(
+                                           os.path.join(self.name,
+                                                        os.pardir)))
     
-    def relprefix(self, path):
+    def relprefix(self, path, separator='_'):
         """ Return a “prefix” string based on a file path –
             the actual path separators are replaced with underscores,
             with which the individual path segments are joined, creating
@@ -964,9 +960,9 @@ class Directory(collections.abc.Hashable,
             
             Used internally in the implementation of “Directory.flatten(…)”.
         """
-        return (self.relparent(path) + os.sep).replace(os.sep, '_')
+        return (self.relparent(path) + os.sep).replace(os.sep, separator)
     
-    def flatten(self, destination, suffix=None):
+    def flatten(self, destination, suffix=None, new_suffix=None):
         """ Copy the entire directory tree, all contents included, to a new
             destination path – with all files residing within the same directory
             level.
@@ -981,7 +977,7 @@ class Directory(collections.abc.Hashable,
             all with names like:
             
                 yo_dogg_[0..9].jpg
-                yo_dogg_nodogg_[0.99].png
+                yo_dogg_nodogg_[0..99].png
             
            `flatten(…)` will not overwrite existant directories. Like, if
             you have yourself an instance of Directory, `directory`, and you
@@ -1007,18 +1003,21 @@ class Directory(collections.abc.Hashable,
             # Create destination directory, and list for the results:
             whereto.makedirs()
             results = []
+            searcher = suffix_searcher(suffix)
             # Walk source directory:
-            for root, dirs, files in self.walk(followlinks=True):
-                filenames = suffix and filter(suffix_searcher(suffix), files) \
-                                    or files
-                inputs = (os.path.join(root, filename) \
-                                         for filename in filenames)
-                outputs = ((whereto.subpath(self.relprefix(root) + filename)) \
-                                              for filename in filenames)
-                for infile, outfile in zip(inputs, outputs):
-                    copied = shutil.copy2(infile, outfile, follow_symlinks=True)
-                    assert os.path.exists(copied)
-                    results.append(copied)
+            for root, dirs, files in self.walk():
+                basic_prefix = self.relprefix(root)
+                filenames = tuple(filter(searcher, files))
+                iinputs = (os.path.join(root, filename) for filename in filenames)
+                outputs = ((whereto.subpath(basic_prefix + (new_suffix and \
+                                                    swapext(filename, new_suffix) or \
+                                                            filename))) \
+                                                        for filename in filenames)
+                for iinfile, outfile in zip(iinputs, outputs):
+                    dstfile = shutil.copy2(iinfile, outfile, follow_symlinks=True)
+                    assert os.path.exists(dstfile)
+                    assert os.path.samefile(dstfile, outfile)
+                    results.append(dstfile)
             # Return the destination directory instance and the result list:
             return whereto, results
     
@@ -1076,7 +1075,7 @@ class Directory(collections.abc.Hashable,
                            suffix=zsuf[1:],
                            randomized=True) as ztmp:
             with zipfile.ZipFile(ztmp.name, "w", zmode) as ziphandle:
-                for root, dirs, files in self.walk(followlinks=True):
+                for root, dirs, files in self.walk():
                     ziphandle.write(root, self.relparent(root)) # add directory
                     for filename in files:
                         filepath = os.path.join(root, filename)
@@ -1202,10 +1201,10 @@ class hd(Directory):
 @export
 class TemporaryDirectory(Directory):
     
-    """ It's funny how this code looks, like, 99 percent exactly like the above
+    """ It’s funny how this code looks, like, 99 percent exactly like the above
         TemporaryName class -- shit just works out that way. But this actually
         creates the directory in question; much like filesystem::TemporaryDirectory
-        from libimread, this class wraps tempfile.mkdtemp() and can be used as
+        from `libimread`, this class wraps tempfile.mkdtemp() and can be used as
         a context manager (the C++ orig used RAII).
     """
     
@@ -1338,7 +1337,7 @@ def NamedTemporaryFile(mode='w+b', buffer_size=-1,
                        suffix="tmp", prefix=DEFAULT_PREFIX,
                        directory=None,
                        delete=True):
-    """ Variation on tempfile.NamedTemporaryFile(…), such that suffixes
+    """ Variation on ``tempfile.NamedTemporaryFile(…)``, such that suffixes
         are passed WITHOUT specifying the period in front (versus the
         standard library version which makes you pass suffixes WITH
         the fucking period, ugh).
