@@ -7,18 +7,20 @@ import argparse
 import contextlib
 import decimal
 import io
+import operator
 import os
 
-from clu.constants.consts import λ
+from clu.constants.consts import λ, φ
 from clu.constants.polyfills import long, unicode, numpy
 from clu.constants.polyfills import HashableABC, SequenceABC, Path
 from clu.constants.polyfills import Mapping, MutableMapping
 from clu.exporting import Exporter
 
 from clu.predicates import (isclasstype,
-                            allpyattrs, getpyattr, haspyattr,
-                            pyattr, or_none,
-                            isiterable,
+                            allpyattrs, haspyattr, nopyattr,
+                            isiterable, haslength,
+                            getpyattr, or_none,
+                            pyattr, attrs,
                             tuplize, uniquify,
                             apply_to, predicate_any,
                                       predicate_all)
@@ -31,16 +33,18 @@ export = exporter.decorator()
 # TYPELISTS: lists containing only types -- according to `clu.predicates.isclasstype(…)` –
 # can be formulated and tested by these lambdas and functions
 
-isunique = lambda thing: isiterable(thing) and (len(frozenset(thing)) == len(tuple(thing)))
+samelength = lambda a, b: haslength(a) and haslength(b) and operator.eq(len(a), len(b))
+differentlength = lambda a, b: haslength(a) and haslength(b) and operator.ne(len(a), len(b))
+isunique = lambda thing: isiterable(thing) and samelength(tuple(thing), frozenset(thing))
 istypelist = predicate_all(isclasstype)
 maketypelist = apply_to(lambda thing: isclasstype(thing) and thing or type(thing),
                         lambda total: tuple(frozenset(total)))
 
 @export
 def isderivative(putative, thing):
-    """ isderivative(thing) → Boolean predicate, True if putative is either a subclass
-        or an instance of thing – depending on whether putative is either a classtype
-        or an instance, basically.
+    """ isderivative(putative, thing) → Boolean predicate, True if putative is either
+        a subclass or an instance of thing – depending on whether putative is either
+        a classtype or an instance, basically.
         
         Used internally to implement `subclasscheck(…)` (née “graceful_issubclass(…)”).
     """
@@ -49,13 +53,13 @@ def isderivative(putative, thing):
     except TypeError:
         return isinstance(putative, thing)
 
-# PREDICATE FUNCTION “graceful_issubclass”: A wrapper for `issubclass(…)`
-# and `isinstance(…)` that tries to work with you, instead of barfing up
-# all sorts of TypeErrors willy-nilly at the slightest misconfiguration: 
+# PREDICATE FUNCTION “subclasscheck”: A wrapper for `issubclass(…)` and
+# `isinstance(…)` that tries to work with you – instead of barfing up
+# all sorts of TypeErrors willy-nilly at the slightest misconfiguration:
 
 subclasscheck = lambda putative, *thinglist: predicate_any(
                 lambda thing: isderivative(putative, thing),
-                                      *maketypelist(*thinglist))
+                                                *thinglist)
 
 # LEGACY CODE SUPPORT:
 graceful_issubclass = subclasscheck
@@ -65,12 +69,9 @@ graceful_issubclass = subclasscheck
 
 numeric_types = uniquify(bool, int, long, float, complex, decimal.Decimal)
 array_types = (array.ArrayType, bytearray, memoryview)
-
-array_types += tuplize(or_none(numpy, 'ndarray'),
-                       or_none(numpy, 'matrix'),
-                       hasattr(numpy, 'ma') and or_none(
-                       getattr(numpy, 'ma'),
-                                      'MaskedArray') or None)
+array_types += attrs(numpy, 'ndarray',
+                            'matrix',
+                            'ma.core.MaskedArray')
 
 # N.B. this numpy typelist does *not* include `decimal.Decimal` –
 # and it *does* include `memoryview`:
@@ -102,10 +103,12 @@ callable_types = Λ + (types.BuiltinFunction,
 # circa version 3.7-ish… PyPy seems to omit them as of v7.1.1, under its
 # Python 3.6-compatible interpreter build:
 
-callable_types += tuplize(or_none(types, 'Coroutine'),
-                          or_none(types, 'ClassMethodDescriptor'),
-                          or_none(types, 'MemberDescriptor'),
-                          or_none(types, 'MethodDescriptor'))
+callable_types += attrs(types, 'Coroutine',
+                               'ClassMethodDescriptor',
+                               'MemberDescriptor',
+                               'MethodDescriptor',
+                               'MethodWrapper',
+                               'WrapperDescriptor')
 
 # PREDICATE FUNCTIONS: is<something>() unary-predicates, many of which make use
 # of the aforementioned typelists:
@@ -127,35 +130,42 @@ isscalar = lambda thing: subclasscheck(thing, scalar_types)
 isstring = lambda thing: subclasscheck(thing, string_types)
 isbytes = lambda thing: subclasscheck(thing, bytes_types)
 ismodule = lambda thing: subclasscheck(thing, types.Module)
-isfunction = ΛΛ = lambda thing: isinstance(thing, Λ) or callable(thing)
-islambda = λλ = lambda thing: pyattr(thing, 'lambda_name', 'name', 'qualname') == λ
+isfunction = ΛΛ = lambda thing: isinstance(thing, Λ) and not isclasstype(thing)
+islambda = λλ = lambda thing: pyattr(thing, 'lambda_name', 'name', 'qualname') in (λ, φ)
+iscallable = lambda thing: haspyattr(thing, 'call') and nopyattr(thing, 'code')
 ishashable = lambda thing: isinstance(thing, HashableABC)
 issequence = lambda thing: isinstance(thing, SequenceABC)
 
-ispathtypelist = lambda thinglist: istypelist(thinglist) and predicate_all(ispathtype, *thinglist)
-ispathlist = lambda thinglist: issequence(thinglist) and predicate_all(ispath, *thinglist)
-isvalidpathlist = lambda thinglist: issequence(thinglist) and predicate_all(isvalidpath, *thinglist)
+# Helper predicates for composing sequence-based predicates:
+isxlist = lambda predicate, thinglist: issequence(thinglist) and predicate_all(predicate, thinglist)
+isxtypelist = lambda predicate, thinglist: istypelist(thinglist) and predicate_all(predicate, thinglist)
 
-isnumberlist = lambda thinglist: issequence(thinglist) and predicate_all(isnumber, *thinglist)
-isnumericlist = lambda thinglist: issequence(thinglist) and predicate_all(isnumeric, *thinglist)
-iscomplexlist = lambda thinglist: issequence(thinglist) and predicate_all(iscomplex, *thinglist)
-isarraylist = lambda thinglist: issequence(thinglist) and predicate_all(isarray, *thinglist)
-isscalarlist = lambda thinglist: issequence(thinglist) and predicate_all(isscalar, *thinglist)
-isstringlist = lambda thinglist: issequence(thinglist) and predicate_all(isstring, *thinglist)
-isbyteslist = lambda thinglist: issequence(thinglist) and predicate_all(isbytes, *thinglist)
-ismodulelist = lambda thinglist: issequence(thinglist) and predicate_all(ismodule, *thinglist)
-isfunctionlist = lambda thinglist: issequence(thinglist) and predicate_all(isfunction, *thinglist)
-islambdalist = lambda thinglist: issequence(thinglist) and predicate_all(islambda, *thinglist)
-ishashablelist = lambda thinglist: issequence(thinglist) and predicate_all(ishashable, *thinglist)
-issequencelist = lambda thinglist: issequence(thinglist) and predicate_all(issequence, *thinglist)
+ispathtypelist = predicate_all(lambda thing: isclasstype(thing) and ispathtype(thing))
+ispathlist = predicate_all(ispath)
+isvalidpathlist = predicate_all(isvalidpath)
+
+isnumberlist = predicate_all(isnumber)
+isnumericlist = predicate_all(isnumeric)
+iscomplexlist = predicate_all(iscomplex)
+isarraylist = predicate_all(isarray)
+isscalarlist = predicate_all(isscalar)
+isstringlist = predicate_all(isstring)
+isbyteslist = predicate_all(isbytes)
+ismodulelist = predicate_all(ismodule)
+isfunctionlist = predicate_all(ΛΛ)
+islambdalist = predicate_all(λλ)
+iscallablelist = predicate_all(iscallable)
+ishashablelist = predicate_all(ishashable)
+issequencelist = predicate_all(issequence)
 
 # MODULE EXPORTS:
+export(samelength,      name='samelength',  doc="samelength(a, b) → boolean predicate, True if both `len(a)` and `len(b)` are defined and equal to each other")
+export(differentlength, name='differentlength', doc="differentlength(a, b) → boolean predicate, True if both `len(a)` and `len(b)` are defined, but are unequal")
 export(isunique,        name='isunique',    doc="isunique(thing) → boolean predicate, True if `thing` is an iterable with unique contents")
 export(istypelist,      name='istypelist',  doc="istypelist(thing) → boolean predicate, True if `thing` is a “typelist” – a list consisting only of class types")
 export(maketypelist,    name='maketypelist',
                          doc="maketypelist(iterable) → convert an iterable of unknown things into a uniquified typelist – a list consisting only of class types")
 
-
 export(subclasscheck,   name='subclasscheck',
                          doc="subclasscheck(putative, *cls_or_tuple) → A wrapper for `issubclass(…)` and `isinstance(…)` that tries to work with you`")
 
@@ -166,11 +176,14 @@ export(graceful_issubclass,
 export(numeric_types,   name='numeric_types')
 export(array_types,     name='array_types')
 export(scalar_types,    name='scalar_types')
-export(bytes_types,     name='bytes_types')
 export(string_types,    name='string_types')
+export(bytes_types,     name='bytes_types')
 export(path_classes,    name='path_classes')
 export(path_types,      name='path_types')
 export(file_types,      name='file_types')
+export(dict_types,      name='dict_types')
+export(mapping_types,   name='mapping_types')
+export(mapping_classes, name='mapping_classes')
 export(function_types,  name='function_types')
 export(Λ,               name='Λ')
 export(callable_types,  name='callable_types')
@@ -196,9 +209,14 @@ export(isfunction,      name='isfunction',  doc="isfunction(thing) → boolean p
 export(ΛΛ,              name='ΛΛ',          doc="ΛΛ(thing) → boolean predicate, True if `thing` is of a callable function type")
 export(islambda,        name='islambda',    doc="islambda(thing) → boolean predicate, True if `thing` is a function created with the «lambda» keyword")
 export(λλ,              name='λλ',          doc="λλ(thing) → boolean predicate, True if `thing` is a function created with the «lambda» keyword")
+export(iscallable,      name='iscallable',  doc="iscallable(thing) → boolean predicate, True if `thing` is a callable type (a class with a “__call__” method) or an instance of same\n\n"
+                                                "N.B. this predicate is *NOT* the same as the built-in “callable(…)” predicate")
 export(ishashable,      name='ishashable',  doc="ishashable(thing) → boolean predicate, True if `thing` can be hashed, via the builtin `hash(…)` function")
 export(issequence,      name='issequence',  doc="issequence(thing) → boolean predicate, True if `thing` is a sequence type (e.g. a `tuple` or `list` type)")
 
+export(isxlist,         name='isxlist',     doc="isxlist(predicate, thinglist) → boolean predicate, True if `thinglist` is a sequence of items, all of which match the predicate function")
+export(isxtypelist,     name='isxtypelist', doc="isxtypelist(predicate, thinglist) → boolean predicate, True if `thinglist` is a typelist, all types in which match the predicate function")
+
 export(ispathtypelist,  name='ispathtypelist',  doc="ispathtypelist(thinglist) → boolean predicate, True if `thinglist` is a sequence of path-related class types")
 export(ispathlist,      name='ispathlist',      doc="ispathlist(thinglist) → boolean predicate, True if `thinglist` is a sequence of path-like instances")
 export(isvalidpathlist, name='isvalidpathlist', doc="isvalidpathlist(thinglist) → boolean predicate, True if `thinglist` is a sequence of valid filesystem path instances")
@@ -213,6 +231,7 @@ export(isbyteslist,     name='isbyteslist',     doc="isbyteslist(thinglist) →
 export(ismodulelist,    name='ismodulelist',    doc="ismodulelist(thinglist) → boolean predicate, True if `thinglist` is a sequence of module types")
 export(isfunctionlist,  name='isfunctionlist',  doc="isfunctionlist(thinglist) → boolean predicate, True if `thinglist` is a sequence of callable function types")
 export(islambdalist,    name='islambdalist',    doc="islambdalist(thinglist) → boolean predicate, True if `thinglist` is a sequence of functions created with the «lambda» keyword")
+export(iscallablelist,  name='iscallablelist',  doc="iscallablelist(thinglist) → boolean predicate, True if `thinglist` is a sequence of callable types (class types with “__call__” methods or instances of same)")
 export(ishashablelist,  name='ishashablelist',  doc="ishashablelist(thinglist) → boolean predicate, True if `thinglist` is a sequence of things that can be hashed, via the builtin `hash(…)` function")
 export(issequencelist,  name='issequencelist',  doc="issequencelist(thinglist) → boolean predicate, True if `thinglist` is a sequence of sequence types")
 
